# Demonstration of Tkinter Canvas simple animation - PMP 060213


import tkinter as tk                                    # Adds a module to Python to provide drawing and GUI tools.
import tkinter.messagebox as tm                         # Adds a module to Python to allow message boxes to be used.


def main():
    root = tk.Tk()   # Sets up an empty window for use.
    app = App(root)
    root.mainloop()  # This runs the program.
    
    
class App:

    def __init__(self, root):
        root.title('8PMP Canvas')           # Gives a title to the window.
        root.geometry('800x600+50+100')     # Sets the size and starting coordinates of the window.
        self.c = tk.Canvas(root, bg='white', width=800, height=600)  # Puts a white drawing 'canvas' in the window (matching its size).
        self.c.pack()                                                # Draws the canvas on the screen.
        # Bind (i.e. link or connect) the pressing of the left or right arrow keys to the 'moveBat' function above.
        self.c.bind_all('<KeyPress-Left>', self.moveBat)
        self.c.bind_all('<KeyPress-Right>', self.moveBat)
        self.game(root)
        

    def game(self, root):
        self.setvariables()
        self.backgroundchoice()
        self.initialize()
        self.play()


    def setvariables(self):
        
        # Set starting co-ordinates and speed of ball and bat.
        self.ball_x = 20   # Horizontal position of the top left corner of an imaginary rectangle containing the ball.
        self.ball_y = 20   # Vertical position of the top left corner of an imaginary rectangle containing the ball.
        self.ball_s = 50   # Size (diameter) of the ball (in pixels).
        self.ball_dx = 4   # Number of pixels by which the ball's position changes horizontally each time it moves.
        self.ball_dy = 4   # Number of pixels by which the ball's position changes vertically each time it moves.
        self.bat_x = 350   # Horizontal position of the top left corner of the bat rectangle.
        self.bat_y = 550   # Vertical position of the top left corner of the bat rectangle.
        self.bat_sx = 100  # Width of the bat (in pixels).
        self.bat_sy = 10   # Depth of the bat (in pixels).
        self.bat_dx = 0    # Number of pixels by which the bat's position changes horizontally each time it moves.
        self.bat_dy = 0    # Number of pixels by which the bat's position changes vertically each time it moves.

        self.score = 0     # Player's score at the start of the game.


    def backgroundchoice(self):
        # Offer the user the choice of having a background image or not.
        self.backImg = tm.askyesno('Bat and Ball', 'Do you want a background image?')
        if self.backImg == True:
            try:
                self.fname = 'Images/pyramid01.gif'
                f = open(self.fname)
                f.close()
                self.userImage = tk.PhotoImage(file = self.fname)
                self.c.create_image(400,300, image = self.userImage)
            except IOError:
                tm.showinfo('Bat and Ball', 'Background image not found.')


    def initialize(self):

        # Put a red ball and a blue bat on the canvas at the starting coordinates set above.
        self.ball = self.c.create_oval(self.ball_x,self.ball_y, self.ball_x+self.ball_s,self.ball_y+self.ball_s, fill='red', tag='redBall')
        self.bat = self.c.create_rectangle(self.bat_x,self.bat_y, self.bat_x+self.bat_sx,self.bat_y+self.bat_sy, fill='blue', tag='blueBat')

        # Put the score at the bottom left of the canvas.
        self.sc = self.c.create_text(75,575, text='Score : ' + str(self.score), font=('Arial', '14', 'bold'), fill='dark green')

        # Wait for the user to click 'OK' before starting the game.
        tm.showinfo('Bat and Ball', 'Use the left and right\narrow keys\nto move the bat.\n\nPress \'OK\' to start.')


    def play(self):

        # Make the ball and bat move until the ball hits the bottom of the screen or the user ends the program.
        while True:
            if self.ball_x <= 0 or self.ball_x+self.ball_s >= 800:  # Check whether the ball is off the screen to the left or right.
                self.ball_dx = -self.ball_dx                        # If so, reverse the horizontal direction of the ball.
            if self.ball_y <= 0:                                    # Check whether the ball is off the screen to the top.
                self.ball_dy = -self.ball_dy                        # If so, reverse the vertical direction of the ball.
            if self.ball_y+self.ball_s >= 600:                      # Check whether the ball is off the screen to the bottom.
                self.restart = tm.askyesno('Bat and Ball', 'Restart?')
                if self.restart == False:
                    break                   # End the program if the player does not want to restart.
                else:                       # Otherwise, reset the ball to its starting position and zero the score.
                    self.c.delete('redBall')
                    self.ball_x = 20
                    self.ball_y = 20
                    self.ball = self.c.create_oval(self.ball_x,self.ball_y, self.ball_x+self.ball_s,self.ball_y+self.ball_s, fill='red', tag='redBall')
                    self.score = 0
            if (self.ball_y+self.ball_s >= self.bat_y and         # Check whether the bottom of the ball overlaps with the bat.
                    self.ball_y+self.ball_s <= self.bat_y+self.bat_sy and
                    self.ball_x+self.ball_s/2 >= self.bat_x and
                    self.ball_x+self.ball_s/2 <= self.bat_x+self.bat_sx):
                self.ball_dy = -self.ball_dy                      # Reverse the vertical direction of the ball if it hits the bat.
                self.score += 1
            self.ball_x += self.ball_dx                           # Change the ball's x-coordinate (i.e. horizontal position).
            self.ball_y += self.ball_dy                           # Change the ball's y-coordinate (i.e. vertical position).
            self.c.move('redBall', self.ball_dx, self.ball_dy)    # Move the ball to a new position relative to its previous one on the canvas.
            if self.bat_x > 10 and self.bat_dx == -4 or self.bat_x+self.bat_sx < 790 and self.bat_dx == 4:
                self.bat_x += self.bat_dx                         # Change the bat's position if has not reached an edge of the canvas.
                self.c.move('blueBat', self.bat_dx, self.bat_dy)  # Move the bat to a new position relative to its previous one on the canvas.
            self.c.itemconfigure(self.sc, text='Score : ' + str(self.score))
            self.c.after(10)                # Wait for 10 milliseconds (i.e. 0.01 seconds).
            self.c.update()                 # Update the canvas to show the ball and bat in their new positiona.


    # Create a function to be used when the user presses <Left> or <Right> to move the bat.
    def moveBat(self, event):
        if event.keysym == 'Left':
            self.bat_dx = -4
        elif event.keysym == 'Right':
            self.bat_dx = 4


if __name__ == '__main__':
    main()
